"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = void 0;
const readStorybookConfig_1 = require("../shared/config/readStorybookConfig");
const validatePluginConfig_1 = require("../shared/config/validatePluginConfig");
const createRollupConfig_1 = require("./rollup/createRollupConfig");
const buildAndWrite_1 = require("./rollup/buildAndWrite");
const createManagerHtml_1 = require("../shared/html/createManagerHtml");
const createPreviewHtml_1 = require("../shared/html/createPreviewHtml");
const findStories_1 = require("../shared/stories/findStories");
async function buildPreview(params) {
    var _a;
    const { pluginConfig, outputDir, rootDir, mainJs, mainJsPath, previewJsPath } = params;
    const { storyImports, storyFilePaths } = await findStories_1.findStories(rootDir, mainJsPath, mainJs.stories);
    const previewHtml = createPreviewHtml_1.createPreviewHtml(rootDir, pluginConfig, previewJsPath, storyImports);
    let config = createRollupConfig_1.createRollupConfig({
        outputDir,
        indexFilename: 'iframe.html',
        indexHtmlString: previewHtml,
        storyFilePaths,
    });
    console.log('main.js', mainJs);
    if (mainJs.rollupConfig) {
        config = (_a = (await mainJs.rollupConfig(config))) !== null && _a !== void 0 ? _a : config;
    }
    await buildAndWrite_1.buildAndWrite(config);
}
async function buildManager(params) {
    const managerHtml = createManagerHtml_1.createManagerHtml(params.mainJs);
    const config = createRollupConfig_1.createRollupConfig({
        outputDir: params.outputDir,
        indexFilename: 'index.html',
        indexHtmlString: managerHtml,
    });
    await buildAndWrite_1.buildAndWrite(config);
}
async function build(params) {
    const { outputDir } = params;
    validatePluginConfig_1.validatePluginConfig(params);
    const { mainJs, mainJsPath, previewJsPath } = readStorybookConfig_1.readStorybookConfig(params);
    await buildManager({ outputDir, mainJs });
    await buildPreview({
        pluginConfig: params,
        outputDir,
        rootDir: process.cwd(),
        mainJs,
        mainJsPath,
        previewJsPath,
    });
}
exports.build = build;
//# sourceMappingURL=build.js.map