"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.storybookPlugin = void 0;
const dev_server_core_1 = require("@web/dev-server-core");
const createManagerHtml_1 = require("../shared/html/createManagerHtml");
const createPreviewHtml_1 = require("../shared/html/createPreviewHtml");
const readStorybookConfig_1 = require("../shared/config/readStorybookConfig");
const validatePluginConfig_1 = require("../shared/config/validatePluginConfig");
const findStories_1 = require("../shared/stories/findStories");
const transformMdxToCsf_1 = require("../shared/mdx/transformMdxToCsf");
const injectExportsOrder_1 = require("../shared/stories/injectExportsOrder");
function storybookPlugin(pluginConfig) {
    validatePluginConfig_1.validatePluginConfig(pluginConfig);
    const { mainJs, mainJsPath, previewJsPath } = readStorybookConfig_1.readStorybookConfig(pluginConfig);
    let config;
    let storyImports = [];
    let storyFilePaths = [];
    return {
        name: 'storybook',
        serverStart(args) {
            config = args.config;
        },
        resolveMimeType(context) {
            if (context.path.endsWith('.mdx')) {
                return 'js';
            }
        },
        async transform(context) {
            const filePath = dev_server_core_1.getRequestFilePath(context, config.rootDir);
            if (context.path.endsWith('.mdx')) {
                context.body = await transformMdxToCsf_1.transformMdxToCsf(context.body, filePath);
                // fall through to below to inject exports order as well
            }
            if (storyFilePaths.includes(filePath)) {
                context.body = await injectExportsOrder_1.injectExportsOrder(context.body, filePath);
            }
        },
        async serve(context) {
            if (context.path === '/') {
                return { type: 'html', body: createManagerHtml_1.createManagerHtml(mainJs) };
            }
            if (context.path === '/iframe.html') {
                ({ storyImports, storyFilePaths } = await findStories_1.findStories(config.rootDir, mainJsPath, mainJs.stories));
                return {
                    type: 'html',
                    body: createPreviewHtml_1.createPreviewHtml(config.rootDir, pluginConfig, previewJsPath, storyImports),
                };
            }
        },
    };
}
exports.storybookPlugin = storybookPlugin;
//# sourceMappingURL=storybookPlugin.js.map